/**
 * @file rrpredictor.hpp
 * @author Lars Johannesen <lj@halloffame.dk>
 *
 * @section LICENSE
 * This program is licensed under LGPL3, see LICENSE
 * 
 * @section DESCRIPTION
 * The classes in this file are used for describing characteristics about ecg records
 */
#ifndef ECGLIB_QRSDETECTOR_RRPREDICTOR_LJ_2014_06_06
#define ECGLIB_QRSDETECTOR_RRPREDICTOR_LJ_2014_06_06 1

#include <boost/circular_buffer.hpp>
#include <tuple>

namespace ecglib {
	/**
	* @brief FWD declaration
	*/
	class qrsdetector_config;

	/**
	 * @brief Simple RR predictor class, estimates when a beat should have occured to trigger backsearch
	 */
	class rrpredictor {
		public:
			/**
			 * @brief Constructor for RR predictor
			 *
			 * @param nrr Number of RR intervals to keep in history
			 * @param fs Sampling frequency
			 */
			rrpredictor(const qrsdetector_config &cfg);

		public:
			/**
			 * @brief Reset detector
			 */
			void reset();

			/**
			 * @brief Mark location loc as a detected peak, and update meanrr etc
			 *
			 * @param loc
			 */
			void detectqrs(const unsigned int loc, const double fs);

			/**
			 * @brief Missed detection
			 *
			 * @param loc 
			 */
			void missed(unsigned int loc, const double fs) {
				_nextqrs += _meanrr;
				++_missed;
			}

			/**
			 * @brief Number of missed detections
			 *
			 * @return 
			 */
			unsigned int nmissed() const{
				return _missed;
			}

			/**
			 * @brief Return true if ready for computing next expected qrs
			 *
			 * @return 
			 */
			bool ready() const {
				return _ready;
			}

			/**
			* @brief Set the RRpredictor to be ready (or not ready)
			*
			* @param r Ready
			*/
			void ready(bool r) {
				_ready = r;
			}

			/**
			 * @brief Location of next qrs
			 *
			 * @return 
			 */
			unsigned int nextqrs(const double fs) const {
				unsigned int nextsamp = static_cast<unsigned int>(_nextqrs * fs/1000.);

				return nextsamp;
			}

			/**
			 * @brief Predicted interval for qrs peak
			 *
			 * @param low
			 * @param high
			 */
			std::tuple<
				unsigned int,
				unsigned int>
			predicted_location(const double fs) {
				unsigned int low = static_cast<unsigned int>(_lastqrs * fs/1000.);
				unsigned int high = static_cast<unsigned int>(_nextqrs * fs/1000.);

				return std::make_tuple(low, high);
			}

		private:
			/**
			 * @brief Ready to predict
			 */
			bool _ready;

			/**
			 * @brief First RR interval in buffer?
			 */
			bool _first;

			/**
			 * @brief Number of missed beats
			 */
			unsigned int _missed;

			/**
			 * @brief Number of RR intervals in history
			 */
			unsigned int _nrr;

			/**
			 * @brief Circular buffer of RR intervals
			 */
			boost::circular_buffer<unsigned int> _rrhist;

			/**
			 * @brief Location of last QRS
			 */
			unsigned _lastqrs;

			/**
			 * @brief Location of next QRS
			 */
			unsigned _nextqrs;

			/**
			 * @brief Running mean of RR
			 */
			unsigned int _meanrr;
	};
}

#endif
