%% @file
% pleth_detector_SQI.m
%
% @author Christopher Scully
%
% @section DISCLAIMER
% This software and documentation (the "Software") were developed at the Food
% and Drug Administration (FDA) by employees of the Federal Government in the
% course of their official duties. Pursuant to Title 17, Section 105 of the
% United States Code, this work is not subject to copyright protection and is
% in the public domain. Permission is hereby granted, free of charge, to any
% person obtaining a copy of the Software, to deal in the Software without
% restriction, including without limitation the rights to use, copy, modify,
% merge, publish, distribute, sublicense, or sell copies of the Software or
% derivatives, and to permit persons to whom the Software is furnished to do
% so. FDA assumes no responsibility whatsoever for use by other parties of
% the Software, its source code, documentation or compiled executables, and
% makes no guarantees, expressed or implied, about its quality, reliability,
%  or any other characteristic. Further, use of this code in no way implies
% endorsement by the FDA or confers any advantage in regulatory decisions.
% Although this software can be redistributed and/or modified freely, we
% ask that any derivative works bear some notice that they are derived from
% it, and any modified versions bear some notice that they have been modified.
%
% @section Examples:
%  [onset_time, sqi] = pleth_detector(pleth);
%
% @section See also:
% qrsdetect
%

%%
% @brief Detects the beats in a plethysmographic signals
% @param abp Plethysmographic signal vector (250Hz)
% @return onset_time as steepest slope Beat annotations in ms
% @return sqi signal quality index (1 (good) or 0 (bad))

function [onset_time,sqi] = pleth_detector(pleth)
%pleth_detector Detects the beats in the plethysmographic signal with a quality index
%
% pleth_detector(pleth)
%
% pleth - vector (250 Hz)
%
%                                  DISCLAIMER
% This software and documentation (the "Software") were developed at the Food
% and Drug Administration (FDA) by employees of the Federal Government in the
% course of their official duties. Pursuant to Title 17, Section 105 of the
% United States Code, this work is not subject to copyright protection and is
% in the public domain. Permission is hereby granted, free of charge, to any
% person obtaining a copy of the Software, to deal in the Software without
% restriction, including without limitation the rights to use, copy, modify,
% merge, publish, distribute, sublicense, or sell copies of the Software or
% derivatives, and to permit persons to whom the Software is furnished to do
% so. FDA assumes no responsibility whatsoever for use by other parties of
% the Software, its source code, documentation or compiled executables, and
% makes no guarantees, expressed or implied, about its quality, reliability,
%  or any other characteristic. Further, use of this code in no way implies
% endorsement by the FDA or confers any advantage in regulatory decisions.
% Although this software can be redistributed and/or modified freely, we
% ask that any derivative works bear some notice that they are derived from
% it, and any modified versions bear some notice that they have been modified.
%
% Written by: Christopher Scully
%
% Examples:
%  [idx,sqi] = pleth_detector(pleth);
%
% See also:
% qrsdetect

onset_time = [];
sqi = [];

sr=250;
%low pass filter
st=1000/sr;%sampling time in msec
[b1,a1]=butter(10,10/(250/2.));%butterworth filter coefficients order 10, 10Hz cutoff
PLETHF=filtfilt(b1,a1,pleth); %forwards-backwards filtering
%compute derivatives
DABP=diff(PLETHF);
DABP2=DABP.^2; %max gives rise point, beginning of rise to peak gives onset time
DDABP=diff(DABP2); %onset time from at 1st positive before peak

tshift=round(428.6/st); %indices to shift forward after finding a beat, set at 500 msec
timing=peak_find(DABP,DDABP,tshift,st);

%CHECK IF BOTTOM 5% of RR-intervals have a median of <0.6 the median of the top
%60% of RR-intervals, if so repeat with tshift based on median of top 60% of
%RR-intervals
prLOW=5;
prHIGH=60;
RRcut=0.5;

RRints=diff(timing(:,2)./sr);
plow=prctile(RRints,prLOW);
phigh=prctile(RRints,prHIGH);

if(sum(RRints<plow) == 0 || sum(RRints>phigh) == 0)
    return;
end

medlow=median(RRints(RRints<plow));
medhigh=median(RRints(RRints>phigh));
if medlow<RRcut*medhigh
    tshift=round(1000*RRcut*medhigh/st);
    timing=peak_find(DABP,DDABP,tshift,st);
end

if size(timing,1)>1 %needed in case bp is flatline signal or no pulses detected
    %save pulse characterisitcs
    onset_time=zeros(length(timing),1); %time of onset in sec
    for n=1:length(timing)-1
        onset_time(n)=timing(n,2)./sr;%CHANGED TO STEEPEST SLOPE
    end
    onset_time(n+1)=timing(n+1,2)./sr;%CHANGED TO STEEPEST SLOPE
    onset_time = round((onset_time * 1000) * sr/1000.);
    sqi = ones(length(onset_time),1);
else
    onset_time=[];
    sqi = [];
end

end


function timing=peak_find(DABP,DDABP,tshift,st)
q=200/st; %if first pulse occurs sooner than expected transit time, beat won't be on annotated results
r=1;
timing=zeros(1,3);
minthr = 1e-5;
der_thresh=max([prctile(DABP,90) minthr]);

while q<length(DABP)-(tshift+1)
    %find derivative above noise
    p=find(DABP(q:end)>der_thresh,1,'first');
    %break out if nothing found significant
    if isempty(p)
        break;
    end
    q=p+q-1; %index in signal
    %break out if close to end of signal, will pick up outside loop
    if q+tshift > length(DABP)
        break;
    end
    %find max derivative
    [~,currloc]=max(DABP(q:q+tshift));
    timing(r,1)=currloc+q; %max slope, add 1 because diff removes a point
    %find onset time, look backward from max derivative for start of up
    %slope
    ot=find(DDABP(1:timing(r,1)-2)<0,1,'last');
    timing(r,2)=ot+2;%onset time, add 2 for slope =0, differential removing another point
    r=r+1;
    q=currloc+q+tshift; %shift forward to 500 msec from peak
end
%to handle last beat
p=find(DABP(q:end)>0.1,1,'first');
if ~isempty(p)
    q=p+q-1;
    [~,currloc]=max(DABP(q:end));
    timing(r,1)=currloc+q;
    ot=find(DDABP(1:timing(r,1)-2)<0,1,'last');
    timing(r,2)=ot+2;%onset time
end

end
